@@ -185,20 +185,22 @@ show_help (struct frontend *obj, struct question *q)
 		}
 		free(help);
 	}
-	printf("%s", question_get_text(obj, "debconf/text-help-keystrokes", "KEYSTROKES:"));
-	printf("\n  %c  ", CHAR_HELP);
-	printf("%s", question_get_text(obj, "debconf/text-help-help", "Display this help message"));
+	printf("%s\n", question_get_text(obj, "debconf/text-help-keystrokes", "KEYSTROKES:"));
+	printf(" ");
+	printf(question_get_text(obj, "debconf/text-help-keystroke", "'%c'"), CHAR_HELP);
+	printf(" %s\n", question_get_text(obj, "debconf/text-help-help", "Display this help message"));
 	if (obj->methods.can_go_back (obj, q)) {
-		printf("\n  %c  ", CHAR_GOBACK);
-		printf("%s", question_get_text(obj, "debconf/text-help-goback", "Go back to previous question"));
+		printf(" ");
+		printf(question_get_text(obj, "debconf/text-help-keystroke", "'%c'"), CHAR_GOBACK);
+		printf(" %s\n", question_get_text(obj, "debconf/text-help-goback", "Go back to previous question"));
 	}
 	if (strcmp(q->template->type, "string") == 0 ||
-	    strcmp(q->template->type, "passwd") == 0 ||
+	    strcmp(q->template->type, "password") == 0 ||
 	    strcmp(q->template->type, "multiselect") == 0) {
-		printf("\n  %c  ", CHAR_CLEAR);
-		printf("%s", question_get_text(obj, "debconf/text-help-clear", "Select an empty entry"));
+		printf(" ");
+		printf(question_get_text(obj, "debconf/text-help-keystroke", "'%c'"), CHAR_CLEAR);
+		printf(" %s\n", question_get_text(obj, "debconf/text-help-clear", "Select an empty entry"));
 	}
-	printf("\n");
 	wrap_print(descr);
 	free(descr);
 }
@@ -216,6 +218,8 @@ printlist (struct frontend *obj, struct question *q, const struct choices *choic
 {
 	int choice_min = -1;
 	int num_cols, num_lines;
+	int used_cols = 1;
+	int last_col_height = 0;
 	int i, k, l;
 	int logcount = 0;
 	int *col_width;
@@ -224,6 +228,11 @@ printlist (struct frontend *obj, struct question *q, const struct choices *choic
 	int line = 0, max_len = 0, col = 0;
 	int width = getwidth();
 	char **fchoices = malloc(sizeof(char *) * choices->count);
+	int horiz = 0;
+
+	if (getenv("DEBCONF_TEXT_HORIZ"))
+		horiz = 1;
+
 	i = choices->count;
 	do {
 		i /= 10;
@@ -262,11 +271,22 @@ printlist (struct frontend *obj, struct question *q, const struct choices *choic
 		if (num_cols == 0)
 			break;
 		num_lines = (choices->count - 1) / num_cols + 1;
+		used_cols = (choices->count - 1) / num_lines + 1;
+		last_col_height = choices->count % num_lines;
 		for (i = 0; i < num_cols; i++)
 			col_width[i] = 0;
 		for (i = 0; i < choices->count; i++)
 		{
-			int current_col = i / num_lines;
+			int current_col;
+			if (horiz)
+			{
+				if (!last_col_height || i < last_col_height * num_cols)
+					current_col = i % used_cols;
+				else
+					current_col = ((i - last_col_height * num_cols) % (used_cols-1));
+			}
+			else
+				current_col = i / num_lines;
 			if (strwidth(fchoices[i]) > col_width[current_col])
 			{
 				col_width[current_col] = strwidth(fchoices[i]);
@@ -292,7 +312,15 @@ printlist (struct frontend *obj, struct question *q, const struct choices *choic
 	}
 	for (i = 0; i < choices->count; i++)
 	{
-		strcat(output[line], fchoices[i]);
+		if (horiz)
+		{
+			int j = line * used_cols + col;
+			if (last_col_height && line > last_col_height)
+				j -= line - last_col_height;
+			strcat(output[line], fchoices[j]);
+		}
+		else
+			strcat(output[line], fchoices[i]);
 		if (strwidth(output[line]) > max_len)
 			max_len = strwidth(output[line]);
 		line++;
@@ -989,7 +1017,7 @@ static int text_progress_set(struct frontend *obj, int val)
 	if (new / 10 == last / 10)
 		return DC_OK;
 	last = new;
-	printf("..%d%%", new);
+	printf("... %d%%", new);
 	fflush(stdout);
 
 	return DC_OK;
